import { apiRequest } from './helpers';
import { apiClient } from '../lib/apiClient';
import {GeneralFilter, ExpertsData, ExpertDetails, ExpertScheduler, ExpertSchedulerSession} from '../types/experts';

export const getExpertsList = (locale: string, filter?: GeneralFilter): Promise<ExpertsData> => apiRequest({
    url: '/home/coachsearch1',
    method: 'post',
    data: { ...filter },
    locale
});

export const getExpertById = (id: string, locale: string): Promise<ExpertDetails> => apiRequest({
    url: '/home/coachdetails',
    method: 'post',
    data: { id },
    locale
});

export const getSchedulerByExpertId = async (expertId: string, locale: string, jwt: string) => {
    const response = await apiClient.post(
        '/home/sessionsignupdata',
        { id: expertId },
        {
            headers: {
                'X-User-Language': locale,
                Authorization: `Bearer ${jwt}`
            }
        }
    );

    return response.data as ExpertScheduler || null;
};

export const getSchedulerSession = async (data: { coachId: number, tagId: number, startAtUtc: string, clientComment: string }, locale: string, jwt: string) => {
    const response = await apiClient.post(
        '/home/sessionsignupsubmit',
        data,
        {
            headers: {
                'X-User-Language': locale,
                Authorization: `Bearer ${jwt}`
            }
        }
    );

    return response.data as ExpertSchedulerSession || null;
};
